#!/bin/env python
"""
by Mike Schmitt, Jan 8, 2012
    
This script pulls out the mutation frequencies from a pileup file given as stdin.

Sites with < 20x depth, and sites with clonal mutations (defined as >10% reads mutated), are excluded from analysis by default.

Usage:

cat seq.pileup | python count-muts.py

"""

from optparse import OptionParser
import sys
import re

parser = OptionParser()
parser.add_option("-d", "--depth", action="store", type='int', dest="mindepth", 
                  help="Minimum depth for counting mutations at a site (default = %default)", default=20)
parser.add_option("-c", "--clonality", action="store", type='float', dest="clonal_cutoff",
                  help="Cutoff of mutant reads for scoring a clonal mutation (default = %default)", default=0.1)

(options, args) = parser.parse_args()

f = sys.stdin

#lines = f.readlines()

depths = []

Aseq = 0
AtoT = 0
AtoC = 0
AtoG = 0

Tseq = 0
TtoA = 0
TtoC = 0
TtoG = 0

Cseq = 0
CtoA = 0
CtoT = 0
CtoG = 0

Gseq = 0
GtoA = 0
GtoT = 0
GtoC = 0

#for i,line in enumerate( lines ):
for line in f:
      linebins = line.split()

#remove start line, end line, and N entries
      linebins[4] = linebins[4].replace('$','')    
      linebins[4] = re.sub('\^.','',linebins[4])    
      linebins[4] = linebins[4].replace('N','')
      linebins[4] = linebins[4].replace('n','')

#remove insertions
      linebins[4] = re.sub('\+1.','',linebins[4])    
      linebins[4] = re.sub('\+2..','',linebins[4])
      linebins[4] = re.sub('\+3...','',linebins[4])    
      linebins[4] = re.sub('\+4....','',linebins[4])    
      
#remove deletions
      linebins[4] = linebins[4].replace('*','')    
      linebins[4] = re.sub('-1.','',linebins[4])    
      linebins[4] = re.sub('-2..','',linebins[4])    
      linebins[4] = re.sub('-3...','',linebins[4])    
      linebins[4] = re.sub('-4....','',linebins[4])    

#count depth
      depth = len(linebins[4])
      depths.append([depth])
      
#count point mutations
      if depth < options.mindepth:
            pass
      
      elif ((float(linebins[4].count('T') + linebins[4].count('t') + linebins[4].count('C') + linebins[4].count('c') + linebins[4].count('G') + linebins[4].count('g') + linebins[4].count('A') + linebins[4].count('a')) / depth) > options.clonal_cutoff):
            pass
      
      elif linebins[2] == 'A':
            Aseq += depth
            AtoT += linebins[4].count('T') + linebins[4].count('t')
            AtoC += linebins[4].count('C') + linebins[4].count('c')
            AtoG += linebins[4].count('G') + linebins[4].count('g')

      elif linebins[2] == 'T':
            Tseq += depth
            TtoA += linebins[4].count('A') + linebins[4].count('a')
            TtoC += linebins[4].count('C') + linebins[4].count('c')
            TtoG += linebins[4].count('G') + linebins[4].count('g')
            
      elif linebins[2] == 'C':
            Cseq += depth
            CtoA += linebins[4].count('A') + linebins[4].count('a')
            CtoT += linebins[4].count('T') + linebins[4].count('t')
            CtoG += linebins[4].count('G') + linebins[4].count('g')
            
      elif linebins[2] == 'G':
            Gseq += depth
            GtoA += linebins[4].count('A') + linebins[4].count('a')
            GtoT += linebins[4].count('T') + linebins[4].count('t')
            GtoC += linebins[4].count('C') + linebins[4].count('c')


totalseq = Aseq + Tseq + Cseq + Gseq
totalmut = AtoT + AtoC + AtoG + TtoA + TtoC + TtoG + CtoA + CtoT + CtoG + GtoA + GtoT + GtoC
mutfreq = (float (totalmut) / float(totalseq))

print ""
print "Minimum depth", options.mindepth
print "Maximum clonality", options.clonal_cutoff
print ""
print "A's sequenced", Aseq
print "A to T:", AtoT, ' %.2e' % (float(AtoT) / float(max(Aseq,1)))
print "A to C:", AtoC, ' %.2e' % (float(AtoC) / float(max(Aseq,1)))
print "A to G:", AtoG, ' %.2e' % (float(AtoG) / float(max(Aseq,1)))
print ""
print "T's sequenced", Tseq
print "T to A:", TtoA, ' %.2e' % (float(TtoA) / float(max(Tseq,1)))
print "T to C:", TtoC, ' %.2e' % (float(TtoC) / float(max(Tseq,1)))
print "T to G:", TtoG, ' %.2e' % (float(TtoG) / float(max(Tseq,1)))
print ""
print "C's sequenced", Cseq
print "C to A:", CtoA, ' %.2e' % (float(CtoA) / float(max(Cseq,1)))
print "C to T:", CtoT, ' %.2e' % (float(CtoT) / float(max(Cseq,1)))
print "C to G:", CtoG, ' %.2e' % (float(CtoG) / float(max(Cseq,1)))
print ""
print "G's sequenced", Gseq
print "G to A:", GtoA, ' %.2e' % (float(GtoA) / float(max(Gseq,1)))
print "G to T:", GtoT, ' %.2e' % (float(GtoT) / float(max(Gseq,1)))
print "G to C:", GtoC, ' %.2e' % (float(GtoC) / float(max(Gseq,1)))
print ""
print "Total nucleotides sequenced", totalseq
print ""
print "Total mutations", totalmut
print ""
print "Overall mutation frequency", '%.2e' % mutfreq


