#!/bin/env python
"""
This script gives position-specific mutation frequencies from a tagcounts file given as stdin.

The output is tab-delimited and specifies:
chromosome number, template base, nucleotide position, depth, mutations to T, C, G, A

Usage:

cat seq.pileup | python count-muts.py

Mike Schmitt, Jan 9, 2012
"""

from optparse import OptionParser
import sys
import re
import csv
import string

parser = OptionParser()
parser.add_option("-d", "--depth", action="store", type='int', dest="mindepth", 
                  help="Minimum depth for counting mutations at a site (default = %default)", default=1)
parser.add_option("-n", "--num_muts", action="store", type='int', dest="num_muts", 
                  help="Minimum number of mutations for scoring a site (default = %default)", default=0)
parser.add_option("-C", "--clonality_max", action="store", type='float', dest="clonal_max",
                  help="Maximum clonality to score a mutation (default = %default)", default=1)
parser.add_option("-c", "--clonality_min", action="store", type='float', dest="clonal_min",
                  help="Minimum clonality to score a mutation (default = %default)", default=0)

(options, args) = parser.parse_args()

f = sys.stdin

lines = f.readlines()

chrom=[]
pos=[]
muts = []
depths = []
template =[]
Tcount=[]
Ccount=[]
Gcount=[]
Acount=[]

for i,line in enumerate( lines ):

      linebins = line.split()

#remove start line, end line, and N entries
      linebins[4] = linebins[4].replace('$','')    
      linebins[4] = re.sub('\^.','',linebins[4])    
      linebins[4] = linebins[4].replace('N','')

#remove insertions
      linebins[4] = re.sub('\+1.','',linebins[4])    
      linebins[4] = re.sub('\+2..','',linebins[4])
      linebins[4] = re.sub('\+3...','',linebins[4])    
      linebins[4] = re.sub('\+4....','',linebins[4])    
      
#remove deletions
      linebins[4] = linebins[4].replace('*','')    
      linebins[4] = re.sub('-1.','',linebins[4])    
      linebins[4] = re.sub('-2..','',linebins[4])    
      linebins[4] = re.sub('-3...','',linebins[4])    
      linebins[4] = re.sub('-4....','',linebins[4])    

#count depth
      depth = len(linebins[4])

#count position-specific mutation frequency
      map(str.upper,linebins[4])
    
      if (
      depth < options.mindepth
      or
      ((float(linebins[4].count('T') + linebins[4].count('C') + linebins[4].count('G') + linebins[4].count('A')) / depth) > options.clonal_max)
      or
      ((float(linebins[4].count('T') + linebins[4].count('C') + linebins[4].count('G') + linebins[4].count('A')) / depth) < options.clonal_min)
      or    
      (max(float(linebins[4].count('T')),float(linebins[4].count('C')),float(linebins[4].count('G')),float(linebins[4].count('A'))) < options.num_muts) 
      ):
            pass
      else:
            mut = linebins[4].count('T') + linebins[4].count('C') + linebins[4].count('G') + linebins[4].count('A')
            Tcount.append(linebins[4].count('T'))
            Ccount.append(linebins[4].count('C'))
            Gcount.append(linebins[4].count('G'))
            Acount.append(linebins[4].count('A'))
            chrom.append(linebins[0])
            pos.append(linebins[1])
            template.append(linebins[2])
            depths.append(depth)
            muts.append(mut)
                        
script_output=zip(chrom, template,pos,depths,muts, Tcount, Ccount, Gcount, Acount)

csv_writer = csv.writer(sys.stdout, delimiter='\t')
csv_writer.writerows(script_output)
